package com.engibank.middleware;

import static com.engibank.util.Jwt.verifyJwt;
import static com.engibank.util.Soap.extractWithKey;
import static com.engibank.util.Soap.genErrorReply;

import com.auth0.jwt.exceptions.JWTVerificationException;
import com.engibank.model.User;
import com.engibank.util.Database;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.xml.namespace.QName;
import javax.xml.soap.Node;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

public class AuthKey implements SOAPHandler<SOAPMessageContext> {
  private static final String AUTHKEY = "authKey";
  private static final Logger LOGGER = Logger.getLogger(AuthKey.class.getName());

  @Override
  public Set<QName> getHeaders() {
    Set<QName> header = new HashSet<>();
    header.add(new QName(AUTHKEY));
    return header;
  }

  @Override
  public boolean handleMessage(SOAPMessageContext soapMessageContext) {
    boolean outRequest = (boolean) soapMessageContext.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);

    if (!outRequest) {
      LOGGER.info("Entering key auth handler");

      SOAPMessage message = soapMessageContext.getMessage();

      SOAPHeader header = null;

      try {
        header = message.getSOAPHeader();
      } catch (SOAPException e) {
        LOGGER.log(Level.SEVERE, "Invalid header", e);
        genErrorReply(message, "Invalid header");
      }

      Iterator<Node> headerIterator = header.getChildElements();

      Node authNode = extractWithKey(headerIterator, AUTHKEY);

      if (authNode == null) {
        genErrorReply(message, String.format("Need header with %s node", AUTHKEY));
      }

      SOAPElement authElement = (SOAPElement) authNode;

      Iterator<Node> authChildIterator = authElement.getChildElements();
      Node keyNode = extractWithKey(authChildIterator, "key");

      if (keyNode == null) {
        genErrorReply(message, String.format("Need 'key' as %s child", AUTHKEY));
      }

      assert keyNode != null;

      String jwtKey = keyNode.getValue();

      LOGGER.info(String.format("Detected jwt: %s", jwtKey));

      int uid = -1;
      try {
        uid = verifyJwt(jwtKey);
      } catch (JWTVerificationException e) {
        LOGGER.log(Level.SEVERE, "JWT verification fail", e);
        genErrorReply(message, "Invalid token");
      }

      Database database = new Database();
      EntityManager em = database.getEntityManager();

      User loggedInUser = em.find(User.class, uid);

      if (loggedInUser == null) {
        genErrorReply(message, "Invalid token");
      }

      assert loggedInUser != null;

      LOGGER.info(String.format("Auth user as %s (%s) ",
          loggedInUser.getAccountNumber(),
          loggedInUser.getName()));
      soapMessageContext.put("user", loggedInUser);
      soapMessageContext.setScope("user", MessageContext.Scope.APPLICATION);
    }
    return true;
  }

  @Override
  public boolean handleFault(SOAPMessageContext soapMessageContext) {
    return false;
  }

  @Override
  public void close(MessageContext messageContext) {

  }
}
