package com.engibank.model;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PostLoad;
import javax.persistence.PrePersist;
import javax.persistence.Transient;
import javax.xml.bind.annotation.XmlTransient;

@Entity()
public class Account {

  @Id()
  @Column(nullable = false)
  @GeneratedValue(strategy = GenerationType.AUTO)
  private int id;

  @JoinColumn(nullable = false)
  @ManyToOne(optional = false)
  private User owner;

  @Column(nullable = false)
  private boolean isVirtual = false;

  @Column(nullable = false)
  private int balance = 0;

  @Transient
  private String accountNumber;

  public Account() {
  }

  /**
   * Create new account.
   *
   * @param owner     Owner of account
   * @param isVirtual is account a virtual account
   */
  public Account(User owner, boolean isVirtual) {
    this.owner = owner;
    this.isVirtual = isVirtual;
    this.balance = 0;
  }

  public int getId() {
    return id;
  }

  public void setId(int id) {
    this.id = id;
  }

  @XmlTransient
  public User getOwner() {
    return owner;
  }

  public void setOwner(User owner) {
    this.owner = owner;
  }

  public boolean isVirtual() {
    return isVirtual;
  }

  public void setVirtual(boolean virtual) {
    isVirtual = virtual;
  }

  public int getBalance() {
    return balance;
  }

  public void setBalance(int balance) {
    this.balance = balance;
  }

  @PrePersist
  @PostLoad
  private void calculateAccountNumber() {
    String selfAccountNumber = Integer.toString(this.id);
    String paddedSelfAcc = "000000000000".substring(selfAccountNumber.length()) + selfAccountNumber;

    if (isVirtual) {
      int ownerId = this.owner.getId();
      String ownerAccountNumber = Integer.toString(ownerId);
      String paddedOwnerAcc = "000000000000".substring(ownerAccountNumber.length());
      paddedOwnerAcc += ownerAccountNumber;

      this.accountNumber = paddedOwnerAcc + paddedSelfAcc;
    } else {
      this.accountNumber = paddedSelfAcc;
    }
  }

  public String getAccountNumber() {
    return accountNumber;
  }

  public void setAccountNumber(String accountNumber) {
    this.accountNumber = accountNumber;
  }
}
