package com.engibank.model;

import com.warrenstrange.googleauth.GoogleAuthenticator;
import com.warrenstrange.googleauth.GoogleAuthenticatorKey;
import java.util.List;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;

@Entity()
public class User {

  @Id()
  @Column(nullable = false)
  @GeneratedValue(strategy = GenerationType.AUTO)
  private int id;

  @Column(length = 64, nullable = false)
  private String name;

  @Column(length = 64, nullable = false)
  private String bank;

  @Column(length = 100, nullable = false)
  private String secret;

  @OneToMany(mappedBy = "owner")
  private List<Account> account;

  @Column(nullable = false)
  private boolean isApi = false;

  public User() {

  }

  /**
   * Create new user.
   *
   * @param name    User name
   * @param bank    User bank name
   * @param balance User initial balance
   */
  public User(String name, String bank, int balance) {
    this.name = name;
    this.bank = bank;

    GoogleAuthenticator googleAuth = new GoogleAuthenticator();
    final GoogleAuthenticatorKey key = googleAuth.createCredentials();
    String secret = key.getKey();
    this.secret = secret;
  }

  public int getId() {
    return id;
  }

  public void setId(int id) {
    this.id = id;
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getBank() {
    return bank;
  }

  public void setBank(String bank) {
    this.bank = bank;
  }

  public String getSecret() {
    return secret;
  }

  public void setSecret(String hash) {
    this.secret = hash;
  }

  public boolean isApi() {
    return isApi;
  }

  public void setApi(boolean api) {
    isApi = api;
  }

  public List<Account> getAccount() {
    return account;
  }

  public void setAccount(List<Account> account) {
    this.account = account;
  }

  public Account getPrimaryAccount() {
    return account.stream().filter(acc -> !acc.isVirtual()).findFirst().get();
  }

  public String getAccountNumber() {
    Account primary = getPrimaryAccount();
    return primary.getAccountNumber();
  }
}
