package com.engibank.service;

import com.engibank.model.Account;
import com.engibank.model.User;
import com.engibank.util.Database;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.Resource;
import javax.jws.HandlerChain;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

@HandlerChain(file = "../authKey.xml")
@WebService()
public class AccountService {
  private static final Logger LOGGER = Logger.getLogger(AccountService.class.getName());

  @Resource
  WebServiceContext context;

  /**
   * Generate new virtual account.
   *
   * @return new virtual account number
   */
  @WebMethod
  public String createVirtualAccount() {
    LOGGER.log(Level.FINE, "New VA request");

    MessageContext ctx = context.getMessageContext();
    User loggedinUser = (User) ctx.get("user");

    Database database = new Database();
    EntityManager em = database.getEntityManager();

    em.getTransaction().begin();

    Account acc = new Account(loggedinUser, true);

    em.persist(acc);

    em.getTransaction().commit();

    database.close();

    LOGGER.log(Level.FINE, String.format("Generated new VA number %s", acc.getAccountNumber()));

    return acc.getAccountNumber();
  }

  /**
   * Validate account number exists.
   *
   * @param accountNumber the account number to check
   * @return is account number exists
   */
  @WebMethod
  public boolean validateAccountNumber(
      @XmlElement(required = true)
      @WebParam(name = "accountNumber", mode = WebParam.Mode.IN) String accountNumber)
      throws Exception {
    Database database = new Database();
    EntityManager em = database.getEntityManager();

    int accNum = 0;

    if (accountNumber.length() == 12) {
      accNum = Integer.parseInt(accountNumber);
    } else if (accountNumber.length() == 24) {
      accNum = Integer.parseInt(accountNumber.substring(12, 24));
    } else {
      throw new Exception("Invalid account length");
    }

    TypedQuery<Account> query = em.createQuery(
        "SELECT a FROM Account a WHERE a.id = :acc_num", Account.class);
    query.setParameter("acc_num", accNum);

    Account account = null;

    try {
      account = query.getSingleResult();
    } catch (NoResultException e) {
      LOGGER.log(Level.INFO, "No account found");
    } catch (Exception e) {
      LOGGER.log(Level.SEVERE, "Validation DB fail", e);
    } finally {
      database.close();
    }

    return account != null;
  }
}
