package com.engibank.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTCreator;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTVerificationException;
import com.auth0.jwt.interfaces.DecodedJWT;
import java.time.Instant;
import java.util.Date;

public class Jwt {
  private static final String ENVKEY = "JWT_SECRET";

  /**
   * Generate JWT token for user.
   *
   * @param userId User id of verified user
   * @return JWT token
   */
  public static String generateJwt(int userId, boolean permanent) {
    Algorithm algorithmHS = Algorithm.HMAC256(System.getenv(ENVKEY));

    JWTCreator.Builder builder = JWT.create().withClaim("user", userId);

    if (!permanent) {
      Instant instant = Instant.now();
      Instant expired = instant.plusSeconds(24 * 3600);
      builder.withExpiresAt(new Date(expired.toEpochMilli()));
    }

    String token = builder.sign(algorithmHS);

    return token;
  }

  /**
   * Verify and decode JWT.
   *
   * @param token JWT token input
   * @return Id of decoded user
   * @throws JWTVerificationException When JWT is invalid
   */
  public static int verifyJwt(String token) throws JWTVerificationException {
    Algorithm algorithmHS = Algorithm.HMAC256(System.getenv(ENVKEY));
    JWTVerifier verifier = JWT.require(algorithmHS).build();

    DecodedJWT verified = verifier.verify(token);
    return verified.getClaim("user").asInt();
  }
}
