package com.engibank.util;

import java.util.Iterator;
import java.util.logging.Logger;
import javax.xml.soap.Node;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.soap.SOAPFaultException;

public class Soap {
  private static final Logger LOGGER = Logger.getLogger(Soap.class.getName());

  /**
   * Extract child of a Node with specific key.
   *
   * @param itemIterator Iterator of child node
   * @param key          Key to search
   * @return Node with specified key or null
   */
  public static Node extractWithKey(Iterator<Node> itemIterator, String key) {
    Node selectedNode = null;

    while (itemIterator.hasNext()) {
      Node tempNode = itemIterator.next();
      if (tempNode.getNodeType() == Node.ELEMENT_NODE) {
        String nodeName = tempNode.getLocalName();
        LOGGER.fine(String.format("Detected node with key %s", nodeName));

        if (nodeName.equals(key)) {
          LOGGER.fine("Key detected");
          selectedNode = tempNode;
        }
      }
    }

    return selectedNode;
  }

  /**
   * Generate and throw SOAP error for SOAPHandler.
   *
   * @param message The original message
   * @param reason  Reason for fault
   */
  public static void genErrorReply(SOAPMessage message, String reason) {
    try {
      SOAPBody soapBody = message.getSOAPPart().getEnvelope().getBody();
      SOAPFault soapFault = soapBody.addFault();
      soapFault.setFaultString(reason);
      throw new SOAPFaultException(soapFault);
    } catch (SOAPException e) {
      e.printStackTrace();
    }
  }

}
